from ._yaxis import YAxis
from plotly.graph_objs.layout import yaxis
from ._xaxis import XAxis
from plotly.graph_objs.layout import xaxis
from ._updatemenu import Updatemenu
from plotly.graph_objs.layout import updatemenu
from ._titlefont import Titlefont
from ._ternary import Ternary
from plotly.graph_objs.layout import ternary
from ._template import Template
from plotly.graph_objs.layout import template
from ._slider import Slider
from plotly.graph_objs.layout import slider
from ._shape import Shape
from plotly.graph_objs.layout import shape
from ._scene import Scene
from plotly.graph_objs.layout import scene
from ._radialaxis import RadialAxis
from ._polar import Polar
from plotly.graph_objs.layout import polar
from ._modebar import Modebar
from ._margin import Margin
from ._mapbox import Mapbox
from plotly.graph_objs.layout import mapbox
from ._legend import Legend
from plotly.graph_objs.layout import legend
from ._image import Image
from ._hoverlabel import Hoverlabel
from plotly.graph_objs.layout import hoverlabel
from ._grid import Grid
from plotly.graph_objs.layout import grid
from ._geo import Geo
from plotly.graph_objs.layout import geo
from ._font import Font
from ._annotation import Annotation
from plotly.graph_objs.layout import annotation
from ._angularaxis import AngularAxis
